#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl
# index.cgi
use HTML::Template;
use strict;
use PureMessage::Config qw(%config);
use Data::Dump qw(dump);
use POSIX qw(mktime ceil);
use warnings;
use URI::Escape 'uri_escape';

use PureMessage::Manager::Request qw($REQUEST %in %text);
use PureMessage::Manager::Response::Quarantine;
use PureMessage::Manager::Lang;
use PureMessage::Manager::Widget;
use PureMessage::MIME::Header qw(header_utf8);
use ActiveState::UTF8 qw(maybe_decode_utf8);

use HTML::Entities;
use ActiveState::I18N::UTF8 qw(utf8_encode utf8_header);
use PureMessage::UTF8::Config qw(%utf8_config);

use PureMessage::QMan qw(prepare_msg_preview);

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::Quarantine',
				'html_meta_charset' => 'utf-8',
				'stderr_to_stdout' =>1
                                     );

my $WIDGET = new PureMessage::Manager::Widget;
$r->{debug} =0;

$r->{no_nav} =1;
$r->{no_wrap} =1;
$r->{no_sidebar} =1;
$r->set_handler('html_head',sub {});
$r->set_handler('body_attrib', sub {});
$r->set_handler('body_start', sub {});
$r->set_handler('body_end',sub {});

# massage the in hash so that it is backwards compatible for the old parse
# routines. 

foreach (keys %in) {
  delete($in{$_}) unless defined($in{$_}) and $in{$_} ne '';
}

our (%in,$in,@notifications);

open STDERR, ">&STDOUT";

my @tabs = (
                {TITLE=>'Content',TAB=>'content' },
                {TITLE=>'Quarantine Info',TAB=>'qinfo'},
                {TITLE=>'Message Source',TAB=>'source'}
           );

# global params
$r->raise_error('error_preview_missing_parameter','message_set_id') unless exists $in{message_set_id};
my $message_set_id = (exists $in{message_set_id}) ? $in{message_set_id} : 0;
$r->raise_error('error_preview_missing_parameter','row') if (!exists $in{row}) and !$r->error();
my $row = $in{row};
my $view = $in{view} || $tabs[0]->{TAB};
my $ms_class = $in{ms_class} || 'PureMessage::Quarantine';
my $ms_class_uri = uri_escape($ms_class);
print_errors(), exit if $r->error(); # stop here if we are missing necessary parameters

my ($mstore,$mset);
{
    eval "require $ms_class";
    $mstore = $ms_class->new;
    $mset = get_mset($mstore);
}

sub print_errors {
    $r->error_dlg() if $r->error();
    $r->print("<p>".notification_str($_)."</p>")  for @notifications;
    $r->send();
}

# find an adjacent non-deleted row
sub find_adjacent_row {
    my $rowid = shift;
    my $direction = shift;
    return -1 unless $mset;
    my $adjacent = -1;    
    while ($adjacent == -1) {
        return $adjacent if ($rowid <= 0 and !$direction);
        if ($direction) {
            $rowid++;
        }
        else {
            $rowid--;
        }
        my $row = $mset->fetch($rowid);
        return $adjacent if ($direction && !$row);
        my $msgid = $row->message_id();
        my $msg = $mstore->message($msgid);
        $adjacent = $rowid if (defined $msg and $msg->folder eq 'cur');
    }
    return $adjacent;
}

if (exists $in{'sm_next'} or exists $in{'sm_prev'}) {
    my $direction = (exists $in{'sm_next'}) ? 1 : 0;
    $row = find_adjacent_row($row, $direction);
}

foreach my $tab (@tabs) {
    $tab->{ROW} = $row;
    $tab->{MESSAGE_SET_ID} = $message_set_id;
    $tab->{MS_CLASS} = $ms_class_uri;
}

# message related functions
sub get_mset {
    my $mstore = shift;
    # try to get a previous mset
    return undef unless $message_set_id;
    return undef if $message_set_id == -1;
    my $mset = $mstore->message_set(id=>$message_set_id);
    $r->print("Unable to open message set $mset") unless $mset;
    return $mset;
}

sub get_first_id {
    my $mset = shift;
    my $i = shift;
    return undef unless $mset;
    my $row = $mset->fetch($i);
    return undef unless $row;
    return $row->message_id;
}

# get message
my ($msg,$msg_id);
{    
    if ($message_set_id == -1) {
        $msg = $mstore->message($row);
    }
    else {
        my $setrow = $mset->fetch($row);
        $msg_id = $setrow->message_id;
        $msg = $mstore->message($msg_id);
    }
}

# display frame
my $template;
my $params = {
    ROW => $row,
    VIEW => $view,
    MESSAGE_SET_ID => $message_set_id,
    MS_CLASS => $ms_class,
    MS_CLASS_URI => $ms_class_uri,
};
if (not exists $in{frame}) {
    $template = HTML::Template->new(filename => 'tmpl/preview/index.html');
}
else {
    FRAME: {
        ($in{frame} eq 'ptop') and do {
            $template = HTML::Template->new(filename => 'tmpl/preview/top.html',);
            $params->{TABS} = \@tabs;
            $params->{VIEW} = $view;
            $params->{ROW} = $row;
            $params->{MESSAGE_SET_ID} = $message_set_id;
            if ($mset) {
                $params->{PREV_LINK} = button({type=>'image', icon=>'previous_d'});
                $params->{NEXT_LINK} = button({type=>'image', icon=>'next_d'});
            }
            #'<img src=/images/next_d.gif>';
            my $prev_id = ($row > 0) ? get_first_id($mset,$in{row}-1) : 0;
            my $next_id = get_first_id($mset, $in{row} + 1) || 0;
            if (find_adjacent_row($row,0) != -1) {
                    $params->{PREV_LINK} = button({type=>'image', icon=>'previous', name=>'sm_prev'});
                }
            if (find_adjacent_row($row,1) != -1) {
                    $params->{NEXT_LINK} = button({type=>'image', icon=>'next', name=>'sm_next'});
                }
            last FRAME;
        };
        ($in{frame} eq 'pbottom') and do {
            $template = HTML::Template->new(filename => 'tmpl/preview/bottom.html');
            last FRAME;
        };
        ($in{frame} eq 'pcenter') and do {
            $template = HTML::Template->new(filename => 'tmpl/preview/center.html');
            $params->{CONTENT} = "Message $msg_id does not seem to be in the quarantine.  It may have been moved or deleted since your query was run.";
            last FRAME if not defined $msg;
            VIEW: {
		require PureMessage::Message::ComputedFields;

                # if this fails we just fall back to the preview of
                # what's available (errors should have been logged)
                my $previewable = prepare_msg_preview($msg);
                my $preview_error = <<EOT;
Unable to retrieve message for preview. Please ensure that the Manager
service is running on your mail filtering server(s) and check the
PureMessage log for more information about the actual cause of the error.
EOT

                ($in{view} eq 'content') and do {
                    if ($previewable) {
                        $params->{CONTENT} = $msg->cleaned_body;
		        # XXX &nbsp; should be decoded into 0xa0, however, 
		        # it will confuse utf8-encoder's guess
		        $params->{CONTENT} =~ s/&nbsp;/ /gi;
		        decode_entities($params->{CONTENT});
		        utf8_encode($params->{CONTENT}, undef, \%utf8_config);
                    }
                    else {
                        $params->{CONTENT} = $preview_error;
                    }
                    $params->{SHOW_HEADERS} = 1;
                    my $no_content = '(none)';
                    for my $element (qw/From To/) {
			$params->{$element} = $msg->header($element) || $no_content;
			utf8_header($params->{$element}, \%utf8_config);
                        $params->{$element} = html_escape($params->{$element});
                    }
                    $params->{Subject} = html_escape($msg->c_subject_utf8) || $no_content;
                    last VIEW;
                    };
                ($in{view} eq 'qinfo') and do {
                    #$params->{CONTENT} = htmlify($msg->minfo->as_hdr());
                    $params->{CONTENT} = "";
                    
                    $params->{SHOW_QINFO} = 1;
                    $params->{QMESSAGE_HSIZE} = bytes_format($msg->header_size);
                    $params->{QMESSAGE_MSIZE} = bytes_format($msg->message_size);
                    $params->{QMESSAGE_ID} = $msg->id;
                    my @items = $msg->minfo_items();
                    #$params->{CONTENT} = dump(@items);
                    my (@tmpl_hdrs, @tmpl_hits, @tmpl_viruses);
                    while (@items) {
                        my ($key, $val) = splice(@items, 0, 2);
                        if ($key !~ /^Spam-Hit/i and $key !~ /^Virus-id/i) {
                            (my $formatted_key = $key) =~ s/-/ /g;
                            push @tmpl_hdrs, {NAME => non_breaking($formatted_key), VALUE => html_escape($val)};
                        }
                    }
                    for my $hit (sort $msg->minfo_value('Spam-Hit')) {
                        my ($name,$value) = split " - ", html_escape($hit);
                        push @tmpl_hits, {NAME => $name, VALUE => $value};
                    }
                    for my $hit (sort $msg->minfo_value('Virus-Id')) {
                        push @tmpl_viruses, {NAME => html_escape($hit)};
                    }                   
                    if (scalar @tmpl_hits) {
                        $params->{SHOW_SPAM_HITS} = 1;
                        $params->{SPAM_HITS}  = \@tmpl_hits;
                    }
                    if (scalar @tmpl_viruses) {
                        $params->{SHOW_VIRUS_HITS} = 1;
                        $params->{VIRUS_HITS}  = \@tmpl_viruses;
                    }                   
                    $params->{QINFO} = \@tmpl_hdrs if scalar @tmpl_hdrs;
                    last VIEW;
                    };
                ($in{view} eq 'source') and do {
                    $params->{s_source_view} = [         
                        {VALUE=>5, SEL=>'selected', LABEL=>'5 KB'},
                        {VALUE=>10, SEL=>'', LABEL=>'10 KB'},
                        {VALUE=>50, SEL=>'', LABEL=>'50 KB'},
                        {VALUE=>100, SEL=>'', LABEL=>'100 KB'},
                        {VALUE=>500, SEL=>'', LABEL=>'500 KB'},
                        {VALUE=>1000, SEL=>'', LABEL=>'1 MB'},
                    ];
                    if (exists $in{s_source_view}) {
                        for my $node (@{$params->{s_source_view}}) {
                            $node->{SEL} = '';
                            $node->{SEL} = 'selected' if $in{s_source_view} eq  $node->{VALUE};
                        }
                    }

		    # If we're running w/ DBMS-based quarantine the
		    # long messages will be truncated and the raw message
		    # preview needs to point this out.
                    $params->{RAW_MSG_TRUNCATED} = 1
			if ($config{quarantine_type} eq "pmdb");

                    $params->{SHOW_SOURCE_SELECT} = 1;
                    if ($previewable) {
		        $params->{CONTENT} = htmlify($msg->raw_body);
                    }
                    else {
                        $params->{CONTENT} = $preview_error;
                    }
                    last VIEW;
                    };    
            }
                if (exists $in{'sm_prev'} or exists $in{'sm_next'}) {
                   $params->{HEADER} .= <<EOT;
<script>
parent.frames.ptop.window.location.href='./preview.cgi?frame=ptop&view=$view&row=$row&message_set_id=$message_set_id&ms_class=$ms_class_uri';
//parent.frames.pbottom.window.location.href='./preview.cgi?frame=pbottom&view=$view&row=$row&message_set_id=$message_set_id';
</script>
EOT
                }

            };
            #$params->{CONTENT} =  "$in<br>".$params->{CONTENT};
            last FRAME;
    }
}

$template->param($params);
$r->add_part($template,'body_data');
$r->send();

__END__
