#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl

# On-demand indexing of parts of the message into the database (centralized
# index). We refer to this as "indexing" since the database itself acts
# as an index.

# Currently handles only indexing (i.e. insertion) of the message body.

use strict;
use PureMessage::Config qw(%config);
use PureMessage::Quarantine;
use PureMessage::MessageStore::pmdb;
use PureMessage::MessageStore::Dir;

use PureMessage::Manager::Request qw(%in);

my $self = "on-demand body indexer";

sub respond_and_exit {
    my $status = shift;
    my $message = shift || '';
    my $separator = "";
    if ($message) {
        $separator = '|';
    }
    print "Content-type: text/plain\n\n";
    print "$status$separator$message\n";
}

# if this is by any chance invoked on a non-pmdb installation we just
# exit with successful status, since this operation is not applicable to
# this kind of setup and there's nothing to do.
unless ($config{'quarantine_type'} eq 'pmdb') {
    respond_and_exit('success');
}

my $global_id;
unless ($global_id = $in{'global_id'}) {
    respond_and_exit('failure', "Invalid or non-existing global id parameter");
}

my $insert_body = $in{'insert_body'};
my $insert_hits = $in{'insert_hits'};
unless ($insert_body or $insert_hits) {
    respond_and_exit('success');
}

# Message store here is actually the centralized index.
# One day we'll sort this out.
my $message_store;
unless ($message_store = PureMessage::Quarantine->new()) {
    respond_and_exit('failure', 'Unable to instantiate PureMessage::Quarantine object');
}

$message_store->log("DEBUG", "$self: Running...");

my ($message, $full_message);
unless (($message = $message_store->message($global_id)) and
       ($full_message = $message->get_local_message())) {
    respond_and_exit('failure', "Unable to instantiate local message object for global id $global_id");
}

eval {
    if ($insert_body) {
        $message_store->log("DEBUG", "$self: Inserting message body...");
        $message_store->do_insert_message_body(message   => $full_message,
                                               global_id => $global_id,
                                               do_commit => 1);
    }
    if ($insert_hits) {
        $message_store->log("DEBUG", "$self: Inserting quarantine rule hits...");
        $message_store->do_insert_rule_hits(message => $full_message,
                                            interactive => 1,
                                            do_commit => 1);
    }
};

if ($@) {
    respond_and_exit('failure', "Error encountered while attempting to insert message body: $@");
}
else {
    respond_and_exit('success');
}
