package PureMessage::Manager::Response::Quarantine;
use strict;
use PureMessage::Manager::Request;
use PureMessage::Manager::Response::Services;
use PureMessage::Manager::Config qw(%config %web_config);
use PureMessage::Version qw($VERSION $PREFIX);
use PureMessage::List;
use PureMessage::Map;
use ActiveState::Bytes qw(bytes_format);
use Exporter;

our  @ISA = ('PureMessage::Manager::Response','Exporter');
our @EXPORT = qw( &bytes_format &htmlify &non_breaking &validate_email &html_escape);


sub new {
    my $proto = shift;
    my $self = $proto->SUPER::new(@_);
    $self->{'html_meta_charset'} = 'utf-8';
    $self->add_taskmenu({ title => 'Tasks',
                          buttons => [[ 
                                        {type=>'text',text=>"View Quarantine Summary", href=>"/Quarantine/index.cgi"},
                                        {type=>'text',text=>"Manage Quarantine", href=>"/Quarantine/Manage/index.cgi"},
                                     ]], 
                        });
    my @buttons =  ( 
                    {type=>'text',text=>"Digest Rules", href=>"/Quarantine/Digest/index.cgi"},
                    {type=>'text',text=>"Digest Options", href=>"/Quarantine/Digest/options.cgi"},
                   ); 
    my $ld = PureMessage::List->Available('digest-users');
    if ($ld) {
        push(@buttons, {type=>'text',text=>$ld->{name},href=>"/Quarantine/edit_list.cgi?id=digest-users"});
    }
    
    $ld = PureMessage::Map->Available('notifications');
    if ($ld) {
        push(@buttons, {type=>'text',text=>$ld->{name},href=>"/Quarantine/edit_map.cgi?id=notifications"});
    }

    $self->add_taskmenu({ title => 'Quarantine Digests',
                        buttons => [\@buttons],
                        });


    return $self;
}


# make plaintext look better in html window
sub htmlify
{
my $tmp = $_[0];
$tmp =~ s/&/&amp;/g;
$tmp =~ s/</&lt;/g;
$tmp =~ s/>/&gt;/g;
$tmp =~ s/\"/&quot;/g;
$tmp =~ s/\'/&#39;/g;
$tmp =~ s/\n/<br>\n/g;
return $tmp;
}

sub non_breaking {
    (my $str = $_[0]) =~ s/\s/&nbsp;/g;
    return $str;
}

sub validate_email {
    return 0 unless $_[0] =~ m{
                            \A
                            \w[-.\w]*
                            \@
                            [-\w]+(?:\.[-\w]+)*
                            \z
                        }x;
    return 1;
}


# html_escape
sub html_escape {
  my $tmp;
  $tmp = $_[0];
  $tmp =~ s/&/&amp;/g;
  $tmp =~ s/</&lt;/g;
  $tmp =~ s/>/&gt;/g;
  $tmp =~ s/\"/&quot;/g;
  $tmp =~ s/\'/&#39;/g;
  return $tmp;
}



1;

