package PureMessage::QDigest::MessageId;
use strict;
require Exporter;
use MIME::Base64 qw(encode_base64 decode_base64);

# Put this in BEGIN to avoid problems when this module
# is loaded before PM::QDigest
BEGIN {
    use base 'Exporter';
    our @EXPORT_OK = qw(encode_id decode_id extract_ids);
}

use PureMessage::QDigest;

my %digest_cfg;

sub encode_id {
    my $id = shift;
    my $store = shift;
    my ($local_id_part, $location_part);
    if ($store->id_is_local($id)) {
        $local_id_part = $id;

        # if the id is local, then we're dealing with local messages and
        # we can get the location part from the message store object itself
        $location_part = $store->location_id();
    } else {
        $local_id_part = $store->local_from_global($id);
        $location_part = $store->location_part($id);
    }
    my $enc = encode_base64(pack('w', $local_id_part), '');
    $enc =~ s/=+$//;
    return "[#$enc-$location_part]";
}

sub decode_id {
    my $encoded_id = shift;
    my $store = shift;
    $encoded_id =~ s/^\[#([^\]]+)\]\z/$1/ || return undef;
    my ($encoded_part, $location_part) = split /-/, $encoded_id;
    $encoded_part .= "=" while length($encoded_part) % 4;
    my $decoded_part = unpack('w', decode_base64($encoded_part));
    if ($location_part) {

        # it's a new style digest id
        return $store->global_from_local($decoded_part, $location_part);
    } else {
        %digest_cfg = PureMessage::QDigest::config()->as_hash()
           unless %digest_cfg;
        if ($digest_cfg{centralized}) {
            # approval of old centralized digests => decode the id into
            # the old long global id; can't decode into current global id
            # since it's the old one that sits in the pending digest
            # (because it's generate by the previous release of the system)
            return $store->gid_from_surrogate_key($decoded_part);
        }
        # otherwise, we're approving based on an old 'local' digest =>
        # decode into the local id
        return $decoded_part;
    }
}

sub extract_ids {
    local $_ = shift;
    my @ids = /(\[#[a-zA-Z0-9+\/-]{2,}\])/g;
}

1;
