package PureMessage::RPC::Response::QMan;
use strict;
use warnings;

use Carp;
use PureMessage::Quarantine;
use PureMessage::QMan;
use base 'PureMessage::RPC::Response';

sub approve {
    my $self = shift;
    my %opts = @_;
    my $id = delete $opts{msgid};

    $self->log("DEBUG", "Message approval request for message $id");
    my $q = PureMessage::Quarantine->new();
    my $msg = $q->message($id);
    unless ($msg) {
	$self->log("ERROR", "Failed to load the message $id");
	$self->raise_error("Failed to load the message $id");
	return 0;
    }
    $self->log("DEBUG", "Loaded message $id OK");
    my $l = $self->_get_logger;
    croak "This object requires a logger" unless $l;

    # XXX Check to see if the 'to' option contains multiple recipients.
    # Unpack it if it does.
    $opts{to} = [ split(/\0/, $opts{to}) ]
	if $opts{to} =~ s/^\0//;

    # call the _local_approve method directly as we are only operating
    # on a single message at a time and we don't want this side to
    # tag the message as approved for any recipients (the calling side
    # takes care of that).
    my $to = PureMessage::QMan::_local_approve($msg, logger => $l, %opts);
    $self->log("INFO", "Message $id approved and sent to: " . join(", ", @$to));

    # It would be nice if the RPC code could just take an array ref and do the
    # right thing with it.
    $self->dispatch(["to"]);
    foreach my $recip (@$to) {
	$self->dispatch([$recip]);
    }
    return 1;
}

# XXX I'd rather the request object tell us explicitly which
# logger object to use when constructing this reponse handler.
sub _get_logger {
    my $self = shift;
    return $self->{logger} ||= $self->{_request}{audit_log};
}

sub log {
    my $self = shift;
    if (my $logger = $self->_get_logger) {
	$logger->log(@_);
    }
}

sub trash {
    my ($self, $id) = @_;

    $self->log("DEBUG", "Message trash request for message $id");

    my $q = PureMessage::Quarantine->new();
    if (my $msg = $q->message($id)) {
	$msg->trash;
	$self->log("DEBUG", "Trashed message $id");
    }
    else {
	$self->log("ERROR", "Message $id missing. Can't perform a local trash");
    }
    return 1;
}

sub forward {
    my ($self, $id, $from, @to) = @_;
    $self->log("DEBUG", "Forwarding message $id to: " . join(", ", @to));
    
    my $q = PureMessage::Quarantine->new();
    my $msg = $q->message($id);
    unless ($msg) {
	my $err = "Message $id missing. Can't forward it to " . join(", ", @to);
	$self->log("ERROR", $err);
	$self->raise_error($err);
	return 0;
    }
    my $l = $self->_get_logger;
    croak "This object requires a logger" unless $l;
    
    PureMessage::QMan::forward($msg, to => \@to, logger => $l);
    $self->log("INFO", "Forwarded message $id to " . join(", ", @to));
    return 1;
}

1;
