package PureMessage::Service::queuerunner;

use strict;
use warnings;
use base 'PureMessage::Service';
use ActiveState::Handy qw(run);
use PureMessage::Config qw(expand_filename);

my $queuerunner_d = expand_filename("etc/init.d/pmx-queue-runner");

sub Available {
    my $class = shift;
    return ($class->new(name => "Queue Runner"));
}

sub start {
    my ($self, %opts) = @_;
    _exec([$queuerunner_d, "start"], %opts);
}

sub stop {
    my ($self, %opts) = @_;
    _exec([$queuerunner_d, "stop"], %opts);
}

sub restart {
    my ($self, %opts) = @_;
    _exec([$queuerunner_d, "restart"], %opts);
}

sub status {
    my %s = (running => 0);
    open my $fh, "$queuerunner_d status |" or
        die "Failed to execute $queuerunner_d: $!";
    chomp(my $status = do { local $/; <$fh> });
    unless (close $fh || ($? >> 8) == 2) {
        die "Failed to get $queuerunner_d status";
    }
    if ($status =~ /pid\s*:\s*(\d+)/s) {
        $s{pid} = $1,
        $s{running} = 1,
        $s{ready} = 1,
    }
    return \%s;        
}

sub running {
    return status()->{running};
}

sub _exec {
    my $cmd = shift;
    my %opts = @_;
    if ($opts{bg}) {
        my $pid = fork;
        die "Can't fork: $!" unless defined $pid;
        return $pid if $pid;
        open STDIN, "</dev/null";
        open STDOUT, ">/dev/null";
        open STDERR, ">/dev/null";
        exec @$cmd;
        die "Failed to exec \@$cmd: $!";
    }
    run('@-' . join(' ', @$cmd) . ' >/dev/null 2>&1');
}

1;
