#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w

eval 'exec /tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w -S $0 ${1+"$@"}'
    if 0; # not running under some shell
#
# Quarantine Digest Approval for PureMessage
# Copyright (C) 2002 ActiveState Corp.
#

use strict;
use pmx_user;
use Digest::MD5 qw(md5_hex);
use Getopt::Long;
use PureMessage::MIME::Parser;
use PureMessage::Logger;
use PureMessage::QMan qw(approve);
use PureMessage::Template qw(expand_template);
use PureMessage::Queue qw(queue_mail);
use PureMessage::Config qw(expand_tmpl_filename);
use PureMessage::Preferences;
use PureMessage::Map;

my %opts;
GetOptions(\%opts,
    'decode-ids',
) or die <<USAGE;
Usage:
  cat foo.msg | pmx-qdigest-approve               # approve messages
  cat foo.msg | pmx-qdigest-approve --decode-ids  # list quarantine ids
USAGE

require PureMessage::Quarantine;
require PureMessage::QDigest;
my $qtype = $PureMessage::Config::config{quarantine_type};
my $qdir = $PureMessage::Config::config{quarantine_dir};
my %config = PureMessage::QDigest::config()->as_hash;
my $mstore = PureMessage::Quarantine->new("$qtype:$qdir", connect => 1);

if ($opts{'decode-ids'}) {
    while (my $l = <>) {
    next if $l =~ /^Subject:/i; # skip the subject line    
    print "$_\n" for map { PureMessage::QDigest::decode_id($_, $mstore) }
			       PureMessage::QDigest::extract_ids($l);
    }
    exit;
}

# Approve messages
my $from = '';
my $digest = '';
my $auto_submitted = '';
my %requested;
my $logger = PureMessage::Logger->new(ident => 'qdigest');

my $mp = PureMessage::MIME::Parser->new;
$mp->decode_te(1);
$mp->handler(part_start => sub {
    my ($p, $hdrs, $ct, $ct_param, $te) = @_;
    if (!$digest and $hdrs =~ /^To:.*\+([0-9a-f]{32})\"?[@%]/msi) {
	$digest = $1;
    }
    if (!$from and $hdrs =~ /^From: (.*)/mi) {
	$from = $1;
    }
    if (!$auto_submitted and $hdrs =~ /^Auto\-Submitted: (.*)/mi) {
	$auto_submitted = $1;
    }
});
my $buf = '';
$mp->handler(data => sub {
    my ($p, $data) = @_;
    $buf .= $data;
    $requested{$_}++ for PureMessage::QDigest::extract_ids($buf);
    $buf =~ s/.*\s+//; # strip any words we know we've seen
});
while (<>) {
    $mp->parse($_);
}
$mp->eof;

$logger->log("INFO", "Approval request from '$from'");

# Compliant with rfc3834.  
# Do not process digest emails that were auto-submitted.
# ie. out of office replies, change of address, challenges, etc.
if ($auto_submitted && ($auto_submitted !~ /^no\s*$/i)) {
    $logger->log("INFO", "Digest reply auto-submitted, not processing");
    exit 0;
}

# Log an error if the MD5 digest wasn't found. This error will not generate a
# reply email to the user.
unless (length $digest) {
    $logger->log("ERR", "approve $from: could not extract MD5 digest");
    exit;
}

# Create an error message, or mail the administrator
my $info = PureMessage::QDigest::retrieve_pending($digest) or do {
    $logger->log("ERR", "approve $from: digest id $digest does not exist");
    reply($from, "ERR", "Could not find record of digest '$digest'");
    exit;
};

unless (%requested) {
    $logger->log("ERR", "approve $from: no digest IDs extracted from message.");
    reply($from, "ERR", "No digest IDs were extracted from the approval request");
    exit;
}

# Approve those messages which are requested, and which are allowed to be
# released by this digest. Warn on illegal values.
my @approved;
for my $id (keys %requested) {
    my $dec;
    # decode_id can die if the ID is invalid.  This can happen 
    # if a non-digest ID is mistakenly extracted from the message.
    eval { $dec = PureMessage::QDigest::decode_id($id, $mstore) };
    if ($@ || !defined $dec) {
        $logger->log("WARNING", "digest ID $id is invalid");
        next;
    }
    next unless $info->{id}{$dec};
    push @approved, $dec;
}

# Barf if none of the quarantine ids were valid. This is most likely somebody
# forging an email.
unless (@approved) {
    my @req = keys %requested;
    $logger->log("ERR", "approve $from: no requested IDs (@req) belong to digest '$digest'");
    reply($from, "ERR", "No requested IDs (@req) belong to digest '$digest'");
    exit;
}

$mstore->message_writable(1);
my @fail;

for my $id (# "Schwartzian Transform"
            map $_->[0],
            sort { $a->[1] <=> $b->[1] }
            map [$_, $mstore->sort_key($_)], @approved) {
    my $msg = $mstore->message($id) or do {
	$logger->log("ERR", "can't open message '$id': $!");
	push @fail, "Cannot approve message '$id': quarantine error";
	next;
    };
    my ($action, $weight) = ($config{approve}{action}, 0);
    for my $r (map lc, $msg->minfo_value('reason')) {
	my ($act, $w);
	if ($config{approve}{$r}) {
	    ($act, $w) = @{$config{approve}{$r}}{qw(action weight)};
	    $w ||= 0;
	}
	else {
	    ($act, $w) = ($config{approve}{action}, 0);
	}
	if ($weight < $w) {
	    $action = $act;
	    $weight = $w;
	}
    }
    $action = $config{approve}{action} unless defined $action;
    $action = 'deny' unless defined $action;
    if ($action eq 'deny') {
	$logger->log("ERR", "approve $from: approving message '$id' for '$info->{to}': DENY");
	push @fail, "Cannot approve message '$id': denied by site policy";
    }
    elsif ($action eq 'admin') {
	$logger->log("ERR", "approve $from: approving message '$id' for '$info->{to}': administrator approval required");
	push @fail, "Cannot approve message '$id': administrator approval required";
    }
    else {
	$logger->log("INFO", "Approving message '$id' for '$info->{to}': queued");
	approve($msg, to => $info->{to}, trash => 0, logger => $logger, quarantine => $mstore);
    }
}
reply($from, "INFO", @fail) if @fail;

sub reply {
    my ($to, $status, @msgs) = @_;

    # Refuse to reply to nobody.
    unless (length $to) {
	$logger->log("ERR", "Can't reply to '$to'");
	exit;
    }

    # map the recipient address to a user ID so we can get that user's
    # preferences.
    my $uid_map = PureMessage::Map->new("recipient-aliases");
    my $uid = $uid_map->transform($to);
    my $pref = PureMessage::Preferences->new;

    # Lookup the approval template based on the user's language preference
    my %config = PureMessage::QDigest::config()->as_hash;
    my $tmpl = expand_tmpl_filename($config{approve_tmpl},
	$pref->language($uid));

    open(my $TEMPLATE, $tmpl) or do {
	$logger->log("ERR", "Can't open error template $tmpl: $!");
	exit;
    };

    my (%directive, @hdr);
    while (<$TEMPLATE>) {
	last if /^$/;
	my ($fname, $fval) = /^([\w-]+)\s*:\s*(.*)/;
	unless ($fname) {
	    $logger->log("ERR", "Bad template header in '$tmpl': $_");
	    exit;
	}
	if ($fname =~ s/X-PMX-//i) {
	    $directive{lc $fname} = $fval;
	}
	else {
	    chomp($fname, $fval);
	    push @hdr, $fname, $fval;
	}
    }

    # Read the template body
    my $template = do { local $/; <$TEMPLATE> };
    close $TEMPLATE;

    # Expand the templates
    for my $txt ($template, @hdr) {
	$txt = expand_template(
	    $txt,
	    vars => {
		user => $to,
		status => $status,
		messages => "   " . join("\n   ", @msgs),
	    },
	);
    }

    my @lines;
    while (@hdr) {
	push @lines, shift(@hdr) . ": " . shift(@hdr);
    }
    push @lines, "", $template;

    # Write the message into the queue
    queue_mail(
	to      => expand_template('%%TO:mail_addr%%', vars => { to => $to }),
	from    => expand_template('%%ADMIN_ADDR:mail_addr%%'),
	message => join("\n", @lines),
    );
}

__END__

=head1 NAME

pmx-qdigest-approve - Queue quarantined messages for approval

=head1 SYNOPSIS

   # As invoked by sendmail to parse approval requests from users. Reads email
   # on standard input.
   | pmx-qdigest-approve

   # To decode quarantine ids read from standard input
   | pmx-qdigest-approve --decode-ids

=head1 DESCRIPTION

Quarantine digests are periodic notifications sent to end-users that summarize
their quarantined messages. By replying to digests, users can automatically
release messages from the quarantine. The replies use an alias, usually
C<pmx-auto-approve>, which is received by the PureMessage server that issued
the quarantine digest. When it receives the message approval requests, the
PureMessage server passes the requests to the C<pmx-qdigest-approve> script,
which processes the release requests.

The C<pmx-qdigest-approve> program can either:

=over 4

=item 1

Parse a user's approval request, adding each approved message to the Queue,
where they await later delivery by the C<pmx-queue> program.

=item 3

Decode quarantine IDs read from standard input, printing one decoded ID per
line.

=back

=head2 Options

=over 4

=item --decode-ids

Used to determine the quarantine ID of a message. When digests are generated,
the original quarantine ID number is obfuscated to prevent messages from
being illegitimately released from quarantine. To determine the message's
quarantine ID from the obfuscated digest ID, use this option.

=back

=head1 LOGGING

Messages approved by C<pmx-qdigest-approve> writes information about the released
message to the C<pmx_log>. The information is encoded using a modified form of
q-encoding (see L<PureMessage::MessageLog> for details). The following fields
are logged:

=over

=item t

The message envelope recipient. This is usually the person who released the
message.

=item id

The message's quarantine ID.

=item f

The envelope sender address.

=item reason

The reason given to the message when it was quarantined.

=item p

The message's spam probability.

=item q

The message's queue ID.

=item r

The IP address of the relay that delivered the message to the PureMessage host.

=item h

A spam rule that was triggered for this message.

=back

Not all fields will be present for all messages.

=head1 SEE ALSO

L<pmx-qdigest>, L<pmx-qdigest-expire>, L<pmx-queue>

=head1 COPYRIGHT

Copyright (C) 2000-2009 Sophos Group.  All rights reserved.  Sophos and
PureMessage are trademarks of Sophos Plc and Sophos Group.

=cut
